@@ -15,7 +15,6 @@ from .const import (
     CONF_OFFLINE,
     CONF_ONLINE,
     CONF_PREFIX,
-    CONF_RELAY,
     CONF_STATE,
     CONF_SWITCHNAME,
     CONF_TOPIC,
@@ -189,16 +188,20 @@ def config_get_friendlyname(config, platform, idx):
     """Get config friendly name."""
     friendly_names = config[CONF_FRIENDLYNAME]
 
-    # If the Tasmota has a relay or light at the index, don't use friendly_names,
-    # instead, switch will be named "binary_sensor <n>"
-    relay_platforms = ["light", "switch"]
-    has_relay_at_idx = config[CONF_RELAY][idx] != 0 and platform not in relay_platforms
-
-    if has_relay_at_idx or idx >= len(friendly_names) or friendly_names[idx] is None:
+    if idx >= len(friendly_names) or friendly_names[idx] is None:
         return f"{config[CONF_DEVICENAME]} {platform} {idx+1}"
     return friendly_names[idx]
 
 
+def config_get_switchfriendlyname(config, platform, idx):
+    """Get config friendly name."""
+    switch_names = config[CONF_SWITCHNAME]
+
+    if idx >= len(switch_names) or switch_names[idx] is None:
+        return f"{config[CONF_DEVICENAME]} {platform} {idx+1}"
+    return switch_names[idx]
+
+
 def config_get_switchname(config, idx):
     """Get switch name."""
     switch_names = config[CONF_SWITCHNAME]
